/**
 * BibleReader.java -  ,   
 */

package jbible.mobile;

import java.util.*;
import java.io.*;
import jbible.data.*;

/**
 *     JAR       jBible.
 * <p>     :     bible  . 
 *   bible      booklist.txt,     
 *    <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/io/DataInput.html#modified-utf-8">
 * </a>  UTF-8      \n.   : 
 * <p>_\t_\t__\n 
 * <p>. .        ,         ,
 *   .
 * <p>      bible/_,       (
 *   1.txt .. _.txt),      content.txt.        
 *     UTF-8,       ,   
 *    \n.      content.txt   :
 * <p>_\t_\n
 * <p>. .               ,  
 * .
 * @author Konstantin K. Beliak, e-mail: <a href="mailto:smbadm@yandex.ru">smbadm@yandex.ru</a>
 */
public class BibleReader implements TextSource {
    
    /**
     *  
     */
    private Book[] books;
    
    /**
     *    
     * @throws java.io.IOException     -      
     */
    public BibleReader() throws IOException {
        String contFileName="/"+Common.BIBLE_TEXT_DIR+"/"+Common.BOOK_LIST_FILE;
        InputStream cont=this.getClass().getResourceAsStream(contFileName);
        if(cont==null) throw new IOException("Book list file not found: "+contFileName);
        DataInputStream dis=new DataInputStream(cont);
        Vector tmpBooks=new Vector();
        int lineNum=0, maxNum=0;
        while(dis.available()>0){
            String book=dis.readUTF().trim();
            int firstTab=book.indexOf('\t');
            int lastTab=book.lastIndexOf('\t');
            if(firstTab==lastTab) throw new IOException("Bad book list format (invalid line format), line "+lineNum+1);
            String numStr=book.substring(0,firstTab).trim();
            String chapCountStr=book.substring(lastTab).trim();
            String text=book.substring(firstTab+1,lastTab).trim();
            int num=0;
            int chapCount=0;
            try{
            	num=Integer.parseInt(numStr);
            	chapCount=Integer.parseInt(chapCountStr);
            }catch(NumberFormatException e){
            	throw new IOException("Bad book list format (invalid number format), line "+lineNum+1);
            }
            if(num<=0) throw new IOException("Bad book list format (invalid book number), line"+lineNum+1);
            if(chapCount<=0) throw new IOException("Bad book list format (invalid chapter count), line "+lineNum+1);
            Book newBook=new Book(num,text);
            String chapterListFile="/"+Common.BIBLE_TEXT_DIR+"/"+num+"/"+Common.CHAPTER_LIST_FILE;
            InputStream inp=this.getClass().getResourceAsStream(chapterListFile);
            if(inp==null) throw new IOException("Bad chapter list file, book "+num);
            DataInputStream chData=new DataInputStream(inp);
            int line=1;
            while(chData.available()>0){
                String str=chData.readUTF().trim();
                int tab=str.indexOf('\t');
                if(tab==-1) throw new IOException("Bad chapter list file (invalid line format), book "+num+", line "+line);
                String strNum=str.substring(0,tab).trim();
                String strCount=str.substring(tab).trim();
                int chNum=0;
                int verseCount=0;
                try{
                    chNum=Integer.parseInt(strNum);
                    verseCount=Integer.parseInt(strCount);
                }catch(NumberFormatException e){
                    throw new IOException("Bad chapter list file (invalid numeric format), book "+num+", line "+line);
                }
                if(chNum<=0) throw new IOException("Bad chapter list file (invalid chapter index), book "+num+", line "+line);
                if(verseCount<=0) throw new IOException("Bad chapter list file (invalid verse count), book "+num+", line "+line);
                if(newBook.getChapter(chNum)!=null) throw new IOException("Bad chapter list file (chapter index "+chNum+" is already used), book "+num+", line "+line);
                Chapter newChapter=newBook.createChapter(chNum);
                newChapter.setVerseCount(verseCount);
                line++;
            }
            chData.close();
            tmpBooks.addElement(newBook);
            if(newBook.getNum()>maxNum) maxNum=newBook.getNum();
            lineNum++;
        }
        dis.close();
        books=new Book[maxNum];
        for(int i=0;i<tmpBooks.size();i++){
            Book curr=(Book)tmpBooks.elementAt(i);
            if(books[curr.getNum()-1]!=null) throw new IOException("Bad book list format (book index "+curr.getNum()+" is already used)");
            books[curr.getNum()-1]=curr;
        }
    }

    /**
     *     
     * @return  
     */
    public Book[] getBibleContents() {
        return books;
    }
    
    /**
     *       
     * @param book   (   0)
     * @return   
     */
    public Chapter[] getBookContents(int book) {
        if(book>0 && book<=books.length){
            Book b=books[book-1];
            if(b==null) return null;
            Chapter[] res=new Chapter[b.getChaptersCount()];
            for(int i=0;i<b.getChaptersCount();i++) res[i]=b.getChapter(i+1);
            return res;
        }else throw new IllegalArgumentException("Bad book number: "+book);
    }

    /**
     *       
     * @param book   (   0)
     * @param chapter   (   0)
     * @return   
     */
    public String[] getChapterVerses(int book, int chapter) {
        String fileName="/"+Common.BIBLE_TEXT_DIR+"/"+book+"/"+chapter+".txt";
        InputStream entry=this.getClass().getResourceAsStream(fileName);
        if(entry==null) return null;
        try{
            DataInputStream dis=new DataInputStream(entry);
            Vector lines=new Vector();
            while(dis.available()>0){
                String s=dis.readUTF().trim();
                lines.addElement(s);
            }
            dis.close();
            String[] res=new String[lines.size()];
            for(int i=0;i<lines.size();i++) res[i]=(String)lines.elementAt(i);
            return res;
        }catch(IOException e){
            return null;
        }
    }
    
}
